//////////////////////////////////////////////////////////////////
// Symbolic
//
// Library for the construction of nested symbolic propositions
// The Flying Pig!
// Started 5/8/2003
// http://www.flypig.co.uk
// 
// 
// The MIT License
// 
// Copyright (c) 2009 David Llewellyn-Jones
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// 
//////////////////////////////////////////////////////////////////

#if !defined _H_SYMBOLIC
#define _H_SYMBOLIC

//////////////////////////////////////////////////////////////////
// Includes

//#include "local.h"

#if defined _RISC_OS
#include "oslib/types.h"
#include "oslib/os.h"
#include "oslib/macros.h"
#endif

#include <stddef.h>
#if !defined (WIN32)
#include <stdbool.h>
#endif

//////////////////////////////////////////////////////////////////
// Defines

//#define _DEBUG
#define _MEM_PROFILE

#ifndef TRUE
#define TRUE                  (1)
#endif
#ifndef FALSE
#define FALSE                 (0)
#endif

#define CONTFRAC_MAXDEPTH     (10)

//////////////////////////////////////////////////////////////////
// Structures
typedef struct _Operation Operation;
typedef struct _Variable Variable;

typedef enum _OPUNARY
{
  OPUNARY_INVALID = -1,

  OPUNARY_NEGATIVE,
  OPUNARY_NOT,
  OPUNARY_VC,
  OPUNARY_COSEC,
  OPUNARY_SINH,
  OPUNARY_COSH,
  OPUNARY_TANH,
  OPUNARY_COS,
  OPUNARY_SIN,
  OPUNARY_TAN,
  OPUNARY_SEC,
  OPUNARY_COT,
  OPUNARY_ARCSIN,
  OPUNARY_ARCCOS,
  OPUNARY_ARCTAN,
  OPUNARY_LN,
  OPUNARY_EXP,
  OPUNARY_ABS,

  OPUNARY_NUM
} OPUNARY;

// TODO: This array should be moved into Convert.cpp
static char const aszOpUnary[OPUNARY_NUM][7] = {
  "-",
  "~",
  "VC",
  "cosec",
  "sinh",
  "cosh",
  "tanh",
  "cos",
  "sin",
  "tan",
  "sec",
  "cot",
  "asin",
  "acos",
  "atan",
  "ln",
  "exp",
  "abs"
};

typedef enum _OPBINARY {
  OPBINARY_INVALID = -1,

  OPBINARY_IND,
  OPBINARY_LAND,
  OPBINARY_LOR,
  OPBINARY_LIMP,
  OPBINARY_POW,
  OPBINARY_ADD,
  OPBINARY_SUB,
  OPBINARY_MUL,
  OPBINARY_DIV,
  OPBINARY_DIVIDE,
  OPBINARY_MOD,
  OPBINARY_OR,
  OPBINARY_AND,
  OPBINARY_BIC,
  OPBINARY_EOR,
  OPBINARY_ROR,
  OPBINARY_ROL,
  OPBINARY_EQ,
  OPBINARY_NE,
  OPBINARY_LT,
  OPBINARY_LE,
  OPBINARY_GT,
  OPBINARY_GE,

  OPBINARY_NUM
} OPBINARY;

// TODO: This array should be moved into Convert.cpp
static char const aszOpBinary[OPBINARY_NUM][4] = {
  "ind",
  "^",
  "v",
  "->",
  "**",
  "+",
  "-",
  "*",
  "DIV",
  "/",
  "MOD",
  "OR",
  "AND",
  "BIC",
  "EOR",
  ">>",
  "<<",
  "=",
  "!=",
  "<",
  "<=",
  ">",
  ">="
};

typedef enum _OPTERNARY {
  OPTERNARY_INVALID = -1,

  OPTERNARY_SET,

  OPTERNARY_NUM
} OPTERNARY;

//////////////////////////////////////////////////////////////////
// Function prototypes

// Memory related functions
void          PropMemReset            (void);
void          PropMemOutput           (void);
void        * PropMemMalloc           (size_t size);
void        * PropMemCalloc           (size_t n, size_t size);
void          PropMemFree             (void * ptr);

// Functions for managing operations
Operation   * CreateMemory            (void);
Operation   * CreateInteger           (int const nInteger);
Operation   * CreateTruthValue        (bool const boTruth);
Operation   * CreateVariable          (char const * szVar);
Operation   * CreateUnary             (OPUNARY eOpType, Operation * psVar1);
Operation   * CreateBinary            (OPBINARY eOpType, Operation * psVar1, Operation * psVar2);
Operation   * CreateTernary           (OPTERNARY eOpType, Operation * psVar1, Operation * psVar2, Operation * psVar3);
char        * OperationToString       (Operation * psOp, char * szString, int nStrLen);
int           OperationToStringLength (Operation * psOp);
char        * OperationToStringC      (Operation * psOp, char * szString, int nStrLen);
int           OperationToStringCLength (Operation * psOp);
Operation   * StringToOperation       (char const * szString);
void          FreeRecursive           (Operation * psOp);
Operation   * CopyRecursive           (Operation * psOp);
Operation   * FindOperation           (Operation * psMain, Operation * psFind);
Operation   * SubstituteOperation     (Operation * psMain, Operation * psFind, Operation * psSub);
Operation   * SubstituteOperationPair (Operation * psMain, Operation * psFind1, Operation * psSub1, Operation * psFind2, Operation * psSub2);
Operation   * SimplifyOperation       (Operation * psOp);
bool          CompareOperations       (Operation * psOp1, Operation * psOp2);
Operation   * ContinuedFraction       (double fDecimal, unsigned int uMaxDepth);
double        ApproximateOperation    (Operation * psOp);
Operation   * UberSimplifyOperation   (Operation * psOp);
Operation   * DifferentiateOperation  (Operation * psOp, Operation * psWRT);

// Functions for managing variables
Variable    * CreateVariableValue     (Operation * psOp, Variable * psVariables);
Variable    * CreateVariables         (Operation * psOp, Variable * psVariables);
Variable    * FindVariable            (Variable * psVariables, char const * const szVar);
void          SetVariable             (Variable * psVar, double fValue);
double        GetVariable             (Variable * psVar);
void          UnsetVariable           (Variable * psVar);
Variable    * FreeVariables           (Variable * psVariables);
int           VariableCount           (Variable * psVariables);
Variable    * VariableFirst           (Variable * psVariables);
Variable    * VariableLast            (Variable * psVariables);
Variable    * VariableNext            (Variable * psVariables);
Variable    * VariablePrev            (Variable * psVariables);
char const *  VariableName            (Variable const * const psVariable);



#endif // if !defined _H_SYMBOLIC
